@@ -333,7 +333,8 @@ void do_one_output_objdump(symbol_collection const & symbols,
 		return ;
 	}
 
-	reader.terminate_process();  // force error code to be acquired
+	// force error code to be acquired
+	reader.terminate_process();
 
 	// required because if objdump stop by signal all above things suceeed
 	// (signal error message are not output through stdout/stderr)
@@ -491,10 +492,27 @@ void output_one_file(istream & in, debug_name_id filename,
 		return;
 	}
 
-	string const out_file = relative_to_absolute_path(output_dir + source);
+	string const out_file = op_realpath(output_dir + source);
 
 	/* Just because you're paranoid doesn't mean they're not out to
-	 * get you ... */
+	 * get you ...
+	 *
+	 * This is just a lame final safety check. If we found the
+	 * source, then "source" should be canonical already, and
+	 * can't escape from the output dir. We can't use op_realpath()
+	 * alone as that needs the file to exist already.
+	 */
+	if (out_file.find("/../") != string::npos) {
+		cerr << "refusing to create non-canonical filename "
+			<< out_file  << endl;
+		return;
+	} else if (!is_prefix(out_file, output_dir)) {
+		cerr << "refusing to create file " << out_file
+		     << " outside of output directory " << output_dir
+		     << endl;
+		return;
+	}
+
 	if (is_files_identical(out_file, source)) {
 		cerr << "input and output files are identical: "
 		     << out_file << endl;
@@ -502,7 +520,7 @@ void output_one_file(istream & in, debug_name_id filename,
 	}
 
 	if (create_path(out_file.c_str())) {
-		cerr << "unable to create directory: "
+		cerr << "unable to create file: "
 		     << '"' << op_dirname(out_file) << '"' << endl;
 		return;
 	}
@@ -532,7 +550,7 @@ string const locate_source_file(debug_name_id filename_id)
 		vector<string>::const_iterator cit = base_dirs.begin();
 		vector<string>::const_iterator end = base_dirs.end();
 		for (; cit != end; ++cit) {
-			string path = relative_to_absolute_path(*cit);
+			string path = op_realpath(*cit);
 
 			if (is_prefix(file, path)) {
 				file = file.substr(path.length());
@@ -545,8 +563,7 @@ string const locate_source_file(debug_name_id filename_id)
 	vector<string>::const_iterator end = search_dirs.end();
 
 	for (; cit != end; ++cit) {
-		string const absfile =
-			relative_to_absolute_path(*cit + "/" + file);
+		string const absfile = op_realpath(*cit + "/" + file);
 
 		if (op_file_readable(absfile)) {
 			return absfile;
@@ -554,8 +571,10 @@ string const locate_source_file(debug_name_id filename_id)
 	}
 
 	/* We didn't find a relocated absolute file, or a relative file,
-	 * assume the original is correct */
-	return origfile;
+	 * assume the original is correct, accounting for the
+	 * possibility it's relative the cwd
+	 */
+	return op_realpath(origfile);
 }
 
 
@@ -601,7 +620,14 @@ bool annotate_source(list<string> const & images)
 
 	if (!output_dir.empty()) {
 
-		output_dir = relative_to_absolute_path(output_dir);
+		if (create_path(output_dir.c_str())) {
+			cerr << "unable to create " << output_dir
+			     << " directory: " << endl;
+			return false;
+		}
+
+		// Make sure we have an absolute path.
+		output_dir = op_realpath(output_dir);
 		if (output_dir.length() &&
 		    output_dir[output_dir.length() - 1] != '/')
 			output_dir += '/';
@@ -612,12 +638,6 @@ bool annotate_source(list<string> const & images)
 				"source files" << endl;
 			return false;
 		}
-
-		if (create_path(output_dir.c_str())) {
-			cerr << "unable to create " << output_dir
-			     << " directory: " << endl;
-			return false;
-		}
 	}
 
 	if (assembly) {
@@ -657,7 +677,8 @@ int opannotate(vector<string> const & non_options)
 	list<string> images;
 
 	list<inverted_profile> iprofiles
-		= invert_profiles(classes, options::extra_found_images);
+		= invert_profiles(options::archive_path, classes,
+				  options::extra_found_images);
 
 	report_image_errors(iprofiles);
 
@@ -666,7 +687,8 @@ int opannotate(vector<string> const & non_options)
 
 	bool debug_info = false;
 	for (; it != end; ++it) {
-		debug_info |= populate_for_image(*samples, *it);
+		debug_info |= populate_for_image(options::archive_path,
+			*samples, *it, options::symbol_filter);
 		images.push_back(it->image);
 	}
 
